require 'dropbox_sdk'

class DbController < ApplicationController
	
	#before_filter :delay # for testing

	def authorize
		# Streamline authentication stages
		unless params[:oauth_token]
			authStart
		else
			authCallback
		end
	end

	def authStart
		# OAuth Step 1: Get a request token from Dropbox
		db_session = DropboxSession.new(DROPBOX_CONFIG["APP_KEY"], DROPBOX_CONFIG["APP_SECRET"])
		begin
			db_session.get_request_token
		rescue DropboxError => e
			render :text => "Exception in OAuth Step 1: #{escape_html e}"
		end

		#serialize and save this DropboxSession
		session[:request_db_session] = db_session.serialize
		
		# OAuth Step 2: Send the user to the Dropbox website so they can authorize
		# our app. After the user authorizes our app, Dropbox will redirect them
		# to our '/oauth-callback' endpoint.

		# pass to get_authorize_url a callback url that will return the user here
		url = db_session.get_authorize_url url_for(:action => 'authorize')

		respond_to do |format|
			format.json { render :json => { :status => :redirect, :url => url } }
			format.html { redirect_to url }
		end
	end

	def authCallback
		# Finish OAuth Step 2: the user has returned from dropbox
		serialized_session = session[:request_db_session]
		unless serialized_session
			# should not ever go here
			render :text => "Error in OAuth step 2: Couldn't find OAuth state in session"
		end
		db_session = DropboxSession.deserialize(serialized_session)

		# OAuth Step 3: Get an access token from Dropbox.
		begin
			db_session.get_access_token
		rescue DropboxError => e
			render :text => "Exception in OAuth step 3: #{escape_html e}"
		end

		session.delete(:request_db_session)
		session[:authorized_db_session] = db_session.serialize
		
		# Track login
		client = get_client()
		user = User.find_or_create_from_hash(client.account_info())
		user.track_login!(request)
		user.save_access_token!(db_session.access_token)

		respond_to do |format|
			format.json { render :json => { :success => true } }
			format.html { redirect_to '' }
		end

		# Consider storing the authorized DropboxSession somewhere more persistent.
		# Instead of in the web session hash.
	end
	
	def logout
		
		#TODO: does this belong here?
		session.delete :authorized_db_session

		respond_to do |format|
			format.json { render :json => { :success => true }}
			format.html { redirect_to '' }
		end
	end
 
	def push
		client = get_client()
		unless client
			return render :json => { :status => :NotAuthorized }
		end
		
		resp = client.put_file(DROPBOX_CONFIG["DB_FILENAME"], request.raw_post, true)

		# Track usage
		user = User.find_or_create_from_hash(client.account_info())
		user.track_usage!

		render :json => { :success => true }
	end 
	
	def pull
		client = get_client()
		unless client
			return render :json => { :status => :NotAuthorized }
		end
		
		begin
			file = client.get_file(DROPBOX_CONFIG["DB_FILENAME"])

			return render :json => { :payload => file }

			rescue DropboxAuthError => e
				# An auth error means the db_session is probably bad
				session.delete :authorized_db_session

				return render :json => { :status => :NotAuthorized }
			
			rescue DropboxError => e
				if e.http_response.code == '404'
					return render :json => { :status => :FileNotFound }
				else
					return render :json => { :status => :DropboxApiError }
				end
		end
	end
	
	private
		def get_client
			if session[:authorized_db_session]
				db_session = DropboxSession.deserialize(session[:authorized_db_session])
				begin
					return DropboxClient.new(db_session, DROPBOX_CONFIG["ACCESS_TYPE"])
				rescue DropboxAuthError => e
					# The stored session didn't work. Fall through and start OAuth
					session[:authorized_db_session].delete
				end
			end
		end
		
		def delay
			sleep(2.0)
		end
end
